package down.to.chat.activities;

import java.util.List;

import down.to.chat.R;
import down.to.chat.connector.AsynchronousConnector;
import down.to.chat.connector.TestConnector;
import down.to.chat.fragments.TransitionalFragment;
import down.to.chat.neighbor.Neighbor;
import down.to.chat.neighbor.NeighborArrayAdapter;
import down.to.chat.neighbor.ViewFragment;
import down.to.chat.photo.PhotoManager;
import down.to.chat.user.User;
import down.to.chat.user.EditFragment;
import android.app.Activity;
import android.content.Intent;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.ListView;

/**
 * Launch activity including user info at the top of the list of neighbors.
 * Neighbors list is backed with arrayadapter for efficiency.
 * 
 * @author Matthew
 *
 */
public class MainUserTopActivity extends Activity {
	
	// reference to currently visible transition fragment
	private TransitionalFragment fragReference;
	private EditFragment editFrag;
	private ViewFragment viewFrag;
	
	// PhotoManager for handling new photos
	private PhotoManager photoManager = new PhotoManager();
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.user_top_layout);
		
		if( findViewById(R.id.user_container) != null) {
			Neighbor user = User.get().initialize(this).
					getNeighborRepresentation();
			
			// editable fragment representation
			if( editFrag == null) {
				editFrag = EditFragment.newInstance(user);
			}
			if( viewFrag == null ) {
				viewFrag = ViewFragment.newInstance(user);
			}
			
			// set nexts to rotate
			editFrag.setNext(viewFrag);
			viewFrag.setNext(editFrag);
			
			// bind fragments as listeners on User
			User.get().bindListener(editFrag);
			User.get().bindListener(viewFrag);
			
			fragReference = viewFrag;
			getFragmentManager().beginTransaction().
				add(R.id.user_container, viewFrag).commit();
			
		}
		
		// associate adapter
		ListView lv = (ListView) findViewById(R.id.neighbors_list);
		NeighborArrayAdapter adapter = new NeighborArrayAdapter(this, 
				R.layout.neighbor_view);
		lv.setAdapter(adapter);
		
		
		// add neighbors to adapter
		AsynchronousConnector connector = new TestConnector(this);
		List<Neighbor> neighbors = connector.getNeighbors();
		adapter.addAll(neighbors);
	}
	
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    // Inflate the menu items for use in the action bar
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.user_top_action_bar, menu);
	    return super.onCreateOptionsMenu(menu);
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	    // Handle presses on the action bar items
	    switch (item.getItemId()) {
	        case R.id.toggle_visible:
	        	toggleVisibilityOfUserFragment(item);
	            return true;
	        case R.id.flip_user:
	        	flipUserFragment(item);
	        	return true;
	        default:
	            return super.onOptionsItemSelected(item);
	    }
	}
	
	/**
	 * Flip the user fragment to the other position if it is currently visible.
	 */
	private void flipUserFragment(MenuItem item) {
		if( !fragReference.isHidden() && fragReference.canTransition() ) {
			Resources res = getResources();
			String edit = res.getString(R.string.menu_edit);
			String save = res.getString(R.string.menu_save);
			
			if( edit.equals(item.getTitle())){
				item.setTitle(save);
			} else if ( save.equals(item.getTitle())) {
				item.setTitle(edit);
			}
			fragReference = fragReference.doTransition(getFragmentManager());
		}
	}
	
	/**
	 * Hide/show the user fragment at the top of the neighbors ui. Updates 
	 * toggle menuitem to say show/hide appropriately.
	 * @param item MenuItem that toggles. 
	 */
	private void toggleVisibilityOfUserFragment(MenuItem item) {
		if( fragReference == null) {
			return;
		}
		
        if( !fragReference.isHidden() ) { 
        	// hide visible fragment
			getFragmentManager().beginTransaction()
			.hide(fragReference)
			.commit();
			
			item.setTitle(getResources().getString(R.string.menu_show));
        } else if( fragReference.isHidden() ) {
        	// show hidden fragment
			getFragmentManager().beginTransaction()
			.show(fragReference)
			.commit();

			item.setTitle(getResources().getString(R.string.menu_hide));
        }
	}

	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, 
			Intent data) {
		if( requestCode == PhotoManager.PHOTO_CROP_ACTION ) {
			if( resultCode == RESULT_OK ){
				photoManager.handleCrop(this, data, editFrag);
			}
		} else if ( requestCode == PhotoManager.PHOTO_TAKE_ACTION ) {
			if( resultCode == RESULT_OK ) {
				photoManager.handleTaken(this, data);
			}
		}
		
	}
}
