package down.to.chat.fragments;

import down.to.chat.R;
import android.app.Fragment;
import android.app.FragmentManager;


/**
 * Interface of fragment which transitions. Ie it can be rotated to anther view.
 * 
 * This always does flip animation.
 * 
 * @author Matthew
 *
 */
public abstract class TransitionalFragment extends Fragment {
	
	// next fragment in transition
	private TransitionalFragment next = null;
	
	/**
	 * Get the next fragment that should replace this one. Returns null if this
	 * fragment should not be transitioned. IE terminal state.
	 * @return Fragment to replace this one, null if no transition.
	 */
	public TransitionalFragment getNext() { 
		return next;
	}
	
	/**
	 * Set the next fragment to return.
	 * @param n next fragment to return.
	 */
	public void setNext(TransitionalFragment n) {
		next = n;
	}
	
	/**
	 * Called before transition. Allows extending classes a hook to take action
	 * before transition takes place.
	 */
	protected void doBeforeTransition() {}
	
	/**
	 * Called after transition. Allows extending classes a hook to take action
	 * after transition takes place but before doTransition returns
	 */
	protected void doAfterTransition() {}
	
	/**
	 * Perform the transition to go to the next fragment.
	 * @param manager FragmentManager used to perform the transition
	 * @return TransitionalFragment transitioned to.
	 */
	public TransitionalFragment doTransition(FragmentManager manager) {
		// do not do transition if getNext is null
		TransitionalFragment next = getNext();
		
		// do precall hook
		doBeforeTransition();
		
		if( next != null ) { 
			manager.beginTransaction()
			.setCustomAnimations(R.animator.card_flip_right_in,
					R.animator.card_flip_right_out,
					R.animator.card_flip_left_in, 
					R.animator.card_flip_left_out)
			.replace(R.id.user_container, next)
			.commit();
		}
		
		// do post hook call
		doAfterTransition();
		return next;
	}

	/**
	 * Check to see if the fragment can transition. We know it cannot transition
	 * if it is still being removed (ie isRemoving) because its removal has 
	 * already started. In addition, if next isRemoving, it must be a live
	 * fragment and cannot be used to replace this.  
	 *  
	 * @return true if the fragment can transition
	 */
	public boolean canTransition() {
		return !isRemoving() && (getNext() != null && !getNext().isRemoving());
	}
}
