package down.to.chat.user;

import java.util.HashSet;
import java.util.Set;

import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Bitmap;

import down.to.chat.enums.Availability;
import down.to.chat.neighbor.Neighbor;

/**
 * Singleton representation of the User. There should only be one user 
 * associated with the app. This provide all the actions to read, update and 
 * store that data.
 * 
 * It also provides hooks to allow listeners to take action on updates.
 * 
 * Any writing to this class should be called from an async runner of some kind
 * since it will involve writing to local settings.
 * 
 * All getters and setters are synchronized to serialize access. In addition, 
 * all listeners are called from within the synchronize block and thus will 
 * be isolated.
 * 
 * @author Matthew
 *
 */
public class User {
	
	// name of shared preferences file and keys
	private final static String PREFERENCES = "down_to_chat_pref";
	private final static String NAME = "dtc_name";
	private final static String NAME_DEFAULT = "Name Unknown";

	// list of listeners for updates on the user
	Set<UserUpdateListener> updateListeners;
	
	// singleton stuff
	private static User user = new User();
	public static User get() { return user; }
	
	private String name;
	private Availability availability;
	private Bitmap photo;
	private Bitmap preview;
	private boolean isInitialized = false;
	
	private User() {
		updateListeners = new HashSet<UserUpdateListener>();
		availability = Availability.OFFLINE;
	}
	
	/**
	 * Pull in name value.
	 * @param context used to pull
	 */
	public User initialize(Context context) {
		if( isInitialized ) {
			return get();
		}
		
		SharedPreferences prefs = context.getSharedPreferences(PREFERENCES, 
				Context.MODE_PRIVATE);
		
		name = prefs.getString(NAME, NAME_DEFAULT);
		
		isInitialized = true;
		return get();
	}
	
	/**
	 * Get a neighbor representation of the user
	 * @return neighbor representation of user.
	 */
	public synchronized Neighbor getNeighborRepresentation() {
		return new Neighbor(name, "null", availability, photo, preview);
	}

	/**
	 * Add a listener to be called on update to User field.
	 * @param listener which will listen on all user fields.
	 */
	public synchronized void bindListener( UserUpdateListener listener ) {
		updateListeners.add(listener);
	}
	
	/**
	 * Remove a listener from list of User's listeners
	 * @param listener to remove
	 * @return true of the listener was removed. false otherwise.
	 */
	public synchronized boolean removeListener( UserUpdateListener listener ) {
		return updateListeners.remove(listener);
	}
	
	/**
	 * @return the name
	 */
	public synchronized String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public synchronized void setName(String name, Context context) {
		if( (this.name == null && name != null) || 
				(this.name != null && !this.name.equals(name))) {
			this.name = name;
			SharedPreferences prefs = context.getSharedPreferences(PREFERENCES, 
					Context.MODE_PRIVATE);
			SharedPreferences.Editor editor = prefs.edit();
			editor.putString(NAME, name);
			editor.apply();
			for(UserUpdateListener listener : updateListeners){
				listener.onNameUpdate(name);
			}
		}
	}

	/**
	 * @return the availability
	 */
	public synchronized Availability getAvailability() {
		return availability;
	}

	/**
	 * @param availability the availability to set
	 */
	public synchronized void setAvailability(Availability availability, 
			Context context) {
		if( this.availability != availability ) {
			this.availability = availability;
			for(UserUpdateListener listener : updateListeners){
				listener.onAvailabilityUpdate(availability);
			}
		}
	}

	/**
	 * @return the photo
	 */
	public synchronized Bitmap getPhoto() {
		return photo;
	}

	/**
	 * @param photo the photo to set
	 */
	public synchronized void setPhoto(Bitmap photo, Bitmap preview, Context context) {
		if((this.photo == null && photo != null) || 
				(this.photo != null && !this.photo.equals(photo))
		   || (this.preview == null && preview != null) || 
			(this.preview != null && !this.preview.equals(photo)) ){
			this.photo = photo;
			this.preview = preview;
			for(UserUpdateListener listener : updateListeners){
				listener.onPhotoUpdate(photo, preview);
			}
		}
	}
	
}
